<?php

require_once (DIR_SYSTEM . "/library/CourierManager/CourierManagerApi.php");

class ControllerShippingCourierManager extends Controller
{
    private $error = array();

    public function index() {
        $this->load->language('shipping/courier_manager');
        $this->document->setTitle($this->language->get('heading_title'));
        $this->load->model('setting/setting');

        if (($this->request->server['REQUEST_METHOD'] == 'POST') && $this->validate()) {
            $this->model_setting_setting->editSetting('courier_manager', $this->request->post);
            $this->session->data['success'] = $this->language->get('text_success');
            $this->redirect($this->url->link("extension/shipping", 'token=' . $this->session->data["token"] . '&type=shipping', true));
        }
        
        $this->data['breadcrumbs'] = array();
        $this->data['breadcrumbs'][] = array(
            'text' => $this->language->get('text_home'),
            'href' => $this->url->link('common/dashboard', 'token='  . $this->session->data["token"], true),
            'separator' => false
        );

        $this->data['breadcrumbs'][] = array(
            'text' => $this->language->get('text_extension'),
            'href' => $this->url->link("extension/shipping", "token=" . $this->session->data["token"] . '&type=shipping', true),
            'separator' => ' :: '
        );

        $this->data['breadcrumbs'][] = array(
            'text' => $this->language->get('heading_title'),
            'href' => $this->url->link('shipping/courier_manager', 'token=' . $this->session->data["token"], true),
            'separator' => ' :: '
        );

        $this->data['action'] = $this->url->link('shipping/courier_manager', 'token=' . $this->session->data["token"], true);
        $this->data['cancel'] = $this->url->link("extension/shipping", 'token=' . $this->session->data["token"] . '&type=shipping', true);

        // Form vars
        $forms = array(
            "courier_manager_api_url", "courier_manager_api_key", "courier_manager_delivery_type",
            "courier_manager_packing_type", "courier_manager_payment_awb", "courier_manager_cod_type",
            "courier_manager_goods_insured", "courier_manager_minimum_amount", "courier_manager_fix_amount",
            "courier_manager_sort_order", "courier_manager_status"
        );
        foreach ($forms as $form) {
            $this->data[$form] = $this->config->get($form);
        }

        // Send post vars to view
        if (!empty($this->request->post)) {
            foreach ($this->request->post as $key => $value) {
                $this->data[$key] = $value;
            }
        }

        $this->data['error'] = $this->error;
        $this->data['heading_title'] = $this->language->get('heading_title');

        $this->template = 'shipping/courier_manager.tpl';
        $this->children = array(
            'common/header',
            'common/footer'
        );

        $this->response->setOutput($this->render());
    }

    public function validate()
    {
        if (!$this->user->hasPermission('modify', 'shipping/courier_manager')) {
            $this->error['warning'] = $this->language->get('error_permission');
        }

        if (!$this->request->post['courier_manager_api_url']) {
            $this->error['api_url'] = "Please fill the API URL";
        }

        if (!$this->request->post['courier_manager_api_key']) {
            $this->error['api_key'] = "Please fill the API KEY";
        }

        if (!$this->request->post['courier_manager_delivery_type']) {
            $this->error['delivery_type'] = "Please fill the Delivery type code";
        }

        if($this->request->post['courier_manager_api_url'] && $this->request->post['courier_manager_api_key']){
            $apiClient = new CourierManagerApi($this->request->post['courier_manager_api_url'], $this->request->post['courier_manager_api_key']);
            $apiClient->getCityList();
            if($apiClient->getError()) {
                $this->error['warning'] = $apiClient->getError();
            }
        }

        return !$this->error;
    }
}