<?php

require_once (DIR_SYSTEM . "/library/CourierManager/CourierManagerApi.php");

class ModelShippingCourierManager extends Model {

    /** @var array */
    private $address;

    /**
     * @return array
     */
    private function getDeliveryData()
    {
        $address = $this->address["address_1"];
        if(!empty($address->address2)) {
            $address.= ", " . $this->address["address_2"];
        }

        // API call data
        $delivery =  array(
            "type" => $this->config->get('courier_manager_delivery_type'),
            "service_type" => $this->config->get('courier_manager_packing_type'),
            "cnt" => $this->cart->countProducts(),
            "weight" => $this->cart->getWeight(),
            "to_country" => $this->address["country"],
            "to_county" => $this->address["zone"],
            "to_city" => $this->address["city"],
            "to_address" => $address,
            "to_zipcode" => $this->address["postcode"]
        );

        return $delivery;
    }

    /**
     * @return int|null
     */
    private function getShippingCost()
    {
        if (is_numeric($this->config->get('courier_manager_minimum_amount')) &&
            $this->cart->getSubTotal() > $this->config->get('courier_manager_minimum_amount')) {
            return 0;
        }

        // Return fix amount if is set
        if (is_numeric($this->config->get('courier_manager_fix_amount')) &&
            $this->config->get('courier_manager_fix_amount') > 0) {
            return $this->config->get('courier_manager_fix_amount');
        }

        // Get API Client
        $apiClient = new CourierManagerApi($this->config->get('courier_manager_api_url'), $this->config->get('courier_manager_api_key'));

        // Call API
        $deliveryData = $this->getDeliveryData();
        $apiPrice = $apiClient->priceAwb($deliveryData);

        // Check if response is ok
        if (!isset($apiPrice->status) || $apiPrice->status != "done") {
            return null;
        }

        // Return delivery price
        return $apiPrice->data->price;
    }

    /**
     * @param array $address
     * @return array
     */
    function getQuote($address) {
        $this->load->language('shipping/courier_manager');
        $this->address = $address;

        $quoteData = array( 'courier_manager' => array(
            'code'         => 'courier_manager.courier_manager',
            'title'        => $this->language->get('text_description'),
            'cost'         => $this->getShippingCost(),
            'tax_class_id' => 0,
            'text'         => $this->currency->format($this->getShippingCost(), $this->session->data['currency'])
        ));

        return array(
            'code'       => 'courier_manager',
            'title'      => $this->language->get('text_title'),
            'quote'      => $quoteData,
            'sort_order' => $this->config->get('courier_manager_sort_order'),
            'error'      => false
        );
    }
}
