<?php

/**
 * Class CourierManager_Shipping_Model_Carrier_CourierManager
 */
class CourierManager_Shipping_Model_Carrier_CourierManager extends Mage_Shipping_Model_Carrier_Abstract implements Mage_Shipping_Model_Carrier_Interface
{
    /** @var string */
    protected $_code = 'CourierManager';

    /** @var CourierManager_Shipping_Model_Carrier_CourierManagerApi */
    protected $apiClient;

    /** @var Mage_Shipping_Model_Rate_Request */
    protected $request;

    /** @var Mage_Sales_Model_Quote */
    protected $quote;

    //
    public function __construct()
    {
        // Init API
        $this->apiClient = new CourierManager_Shipping_Model_Carrier_CourierManagerApi($this->getConfigData('apiurl'), $this->getConfigData('api_key'));

        parent::__construct();
    }

    /**
     * @param Mage_Sales_Model_Order|Mage_Sales_Model_Quote $order
     * @param bool $fromQuote
     * @return mixed
     */
    private function calculateSubTotal($order, $fromQuote = true)
    {
        if ($fromQuote) {
            // Get total key
            $subTotalKey = "grand_total";
            if($this->getMagentoVersion() >= 9) {
                $subTotalKey = "subtotal";
            }

            /** @var Mage_Sales_Model_Quote $quote */
            $quote = $order;
            $totals = $quote->getTotals();
            return $totals[$subTotalKey]->getValue();
        }

        return $order->getSubtotalInclTax();
    }

    /**
     * @param Mage_Sales_Model_Order|Mage_Sales_Model_Quote $order
     * @param bool $fromQuote
     * @return array
     */
    private function getDeliveryData($order, $fromQuote = true)
    {
        // Subtotal
        $subtotal = $this->calculateSubTotal($order, $fromQuote);

        // Weight of all products
        if ($fromQuote) {
            $totalWeight = $order->getShippingAddress()->getWeight();
        }else{
            $totalWeight = $order->getWeight();
        }

        /*
        // Get package Qty
        if ($fromQuote) {
            $packageQty = $this->request->getPackageQty();
        }else{
            $packageQty = 0;
            /** @var Mage_Sales_Model_Order_Item $item * /
            foreach ($order->getAllItems() as $item) {
                $packageQty+=$item->getQtyOrdered();
            }
        }*/

        // Delivery data
        $name = $order->getShippingAddress()->getData('firstname') . " " . $order->getShippingAddress()->getData('lastname');
        $address = str_replace("\n", " ", $order->getShippingAddress()->getData('street'));
        $countryId = $order->getShippingAddress()->getData('country_id');
        $regionId = $order->getShippingAddress()->getData('region_id');

        /** @var Mage_Directory_Model_Country $countryModel */
        $countryModel = Mage::getModel('directory/country')->load($countryId);
        $countryName = $countryModel->getName();

        /** @var Mage_Directory_Model_Region $countryModel */
        $regionModel = Mage::getModel('directory/region')->load($regionId);
        $regionName = $regionModel->getName();

        // API call data
        $priceData = array(
            "use_default_from_address" => "true",
            "type" => $this->getConfigData('expedition_type'),
            "service_type" => $this->getConfigData('type_cod'),
            "cnt" => 1,//$packageQty,
            "weight" => $totalWeight,
            "to_name" => $name,
            "to_contact" => $name,
            "to_phone" => $order->getShippingAddress()->getData('telephone'),
            "to_email" => $order->getShippingAddress()->getData('email'),
            "to_country" => $countryName,
            "to_county" => $regionName,
            "to_city" => $order->getShippingAddress()->getData('city'),
            "to_address" => $address,
            "to_zipcode" => $order->getShippingAddress()->getData('postcode')
        );

        // Add cash on delivery
        if ($order->getPayment()->getMethod() == "cashondelivery" || $fromQuote) {

            $priceData["ramburs"] = $subtotal;
            $priceData["payer"] = "destinatar";

            if ($this->getConfigData('paymentdest') == 0) {
                $priceData["ramburs"] = $priceData["ramburs"]+(float)$order->getShippingAmount()+(float)$order->getShippingTaxAmount();
                $priceData["payer"] = "client";
            }

            $priceData["ramburs_type"] = $this->getConfigData('cod_type');
        }
        return $priceData;
    }

    /**
     * @return int|mixed|null
     */
    private function getShippingCost()
    {
        // Subtotal
        $subtotal = $this->calculateSubTotal($this->quote);

        // Check if total is upper minim free sum
        if (is_numeric($this->getConfigData('minim_free')) && $subtotal > $this->getConfigData('minim_free')) {
            return 0;
        }

        // Return fix amount if is set
        if (is_numeric($this->getConfigData('fix_amount')) && $this->getConfigData('fix_amount') > 0) {
            return $this->getConfigData('fix_amount');
        }

        // Call API
        $deliveryData = $this->getDeliveryData($this->quote);
        $apiPrice = $this->apiClient->priceAwb($deliveryData);

        // Check if response is ok
        if (!isset($apiPrice->status) || $apiPrice->status != "done") {
            return null;
        }

        if ($this->getConfigData('total_shipping_vat') > 0) {
            $taxes = $apiPrice->data->price * ( $this->getConfigData('total_shipping_vat') / 100 );
            return $apiPrice->data->price + $taxes;
        }

        // Return delivery price
        return $apiPrice->data->price;
    }

    /**
     * @param Mage_Shipping_Model_Rate_Request $request
     * @return bool|Mage_Sales_Model_Quote_Address_Rate|Mage_Shipping_Model_Rate_Result|null
     * @throws Mage_Core_Model_Store_Exception
     */
    public function collectRates(Mage_Shipping_Model_Rate_Request $request)
    {
        if (!$this->getConfigFlag('active')) {
            return false;
        }

        // Set request
        $this->request = $request;

        // Current quote
        $this->quote = Mage::getSingleton('checkout/session')->getQuote();
        if(Mage::app()->getStore()->isAdmin())
        {
            $this->quote = Mage::getSingleton('adminhtml/session_quote')->getQuote();
        }

        /** @var Mage_Sales_Model_Quote_Address_Rate $rate */
        $rate = Mage::getModel('shipping/rate_result_method');
        $rate->setCarrier($this->_code);
        $rate->setCarrierTitle($this->getConfigData('title'));
        $rate->setMethod('large');
        $rate->setPrice($this->getShippingCost());

        return $rate;
    }

    /**
     * @param Varien_Event_Observer $observer
     * @throws Exception
     */
    public function orderCheckoutSuccess(Varien_Event_Observer $observer)
    {
        /** @noinspection PhpUndefinedMethodInspection */
        $orderIds = $observer->getEvent()->getOrderIds();
        if (empty($orderIds) || !is_array($orderIds)) {
            return;
        }
        foreach ($orderIds as $order_id) {
            /** @var Mage_Sales_Model_Order $order */
            $order = Mage::getModel('sales/order')->load($order_id);

            if ($order->getShippingCarrier()->getCarrierCode() != $this->_code) {
                return;
            }

            // Call API
            $deliveryData = $this->getDeliveryData($order, false);
            $apiResponse = $this->apiClient->createAwb($deliveryData);

            if (isset($apiResponse->status) && $apiResponse->status == "done" && isset($apiResponse->data->no)) {
                $order->setShippingDescription("{$this->getConfigData('title')} (AWB {$apiResponse->data->no})");
                $order->save();
            }
        }
    }

    /**
     * @return mixed
     */
    protected function getMagentoVersion()
    {
        $version = Mage::getVersionInfo();
        return $version['minor'];
    }

    /**
     * @return array
     */
    public function getAllowedMethods()
    {
        return array($this->_code => $this->getConfigData('title'));
    }

    /**
     * @return bool
     */
    public function isTrackingAvailable()
    {
        return true;
    }
}
