<?php
if (!defined('_PS_VERSION_')) {
    exit;
}

require_once (__dir__ . "/CourierManagerApi.php");

/**
 * Class couriermanager
 */
class Couriermanager extends CarrierModule
{
    const CM_API_URL = "https://app.couriermanager.eu/cscourier";
    const DEBUG = "0";

    const DEFAULT_COUNTRY = "Romania";

    /** @var CourierManagerApi */

    protected $apiClient;

    /** @var CartCore */
    protected $cart;

    /** @var OrderCore */
    protected $order;

    /**
     * Couriermanager constructor.
     */
    public function __construct()
    {
        $this->name = 'couriermanager';
        $this->tab = 'shipping_logistics';
        $this->version = '1.0.5';
        $this->author = 'DevGeeks';
        $this->author_uri = 'https://devgeeks.ro/';
        $this->need_instance = 0;
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('Courier Manager');
        $this->description = $this->l('Courier software that just works.');
        $this->confirmUninstall = $this->l('Are you sure you want to uninstall?');

        $this->ps_versions_compliancy = array(
            'min' => '1.4',
            'max' => _PS_VERSION_
        );

        $this->apiClient = new CourierManagerApi(Configuration::get('CM_API_URL'), Configuration::get('CM_API_KEY'));
    }

    /**
     * @return bool
     * @throws PrestaShopException
     */
    public function install()
    {
        if (_PS_VERSION_ >= 1.5 && Shop::isFeatureActive()) {
            Shop::setContext(Shop::CONTEXT_ALL);
        }

        // Register Carrier
        $carrierConfig = array(
            0 => array('name' => $this->displayName,
                'id_tax_rules_group' => 0,
                'active' => true,
                'deleted' => 0,
                'shipping_handling' => false,
                'range_behavior' => 0,
                'delay' => array('en' => $this->description, Language::getIsoById(Configuration::get('PS_LANG_DEFAULT')) => $this->description),
                'id_zone' => 1,
                'is_module' => true,
                'shipping_external' => true,
                'external_module_name' => $this->name,
                'need_range' => true
            )
        );
        $id_carrier = $this->installExternalCarrier($carrierConfig[0]);
        Configuration::updateValue('COURIERMANAGER_MODULE', (int) $id_carrier);

        if (!parent::install() ||
            !$this->registerHook('actionOrderStatusUpdate') ||
            !Configuration::updateValue('CM_API_URL', self::CM_API_URL) ||
            !Configuration::updateValue('DEBUG', self::DEBUG)
        ) {
            return false;
        }

        return true;
    }

    /**
     * @return bool
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public function uninstall()
    {
        if (!parent::uninstall() || !$this->unregisterHook('actionOrderStatusUpdate'))
            return false;

        // Delete External Carrier
        $carrier = new Carrier((int) (Configuration::get('COURIERMANAGER_MODULE')));
        // If external carrier is default set other one as default
        if (Configuration::get('PS_CARRIER_DEFAULT') == (int) ($carrier->id)) {
            global $cookie;
            $carriersD = Carrier::getCarriers($cookie->id_lang, true, false, false, NULL, PS_CARRIERS_AND_CARRIER_MODULES_NEED_RANGE);
            foreach ($carriersD as $carrierD)
                if ($carrierD['active'] AND !$carrierD['deleted'] AND ($carrierD['name'] != $this->_config['name']))
                    Configuration::updateValue('PS_CARRIER_DEFAULT', $carrierD['id_carrier']);
        }
        // Then delete Carrier
        $carrier->deleted = 1;
        if (!$carrier->update())
            return false;

        return true;
    }

    /**
     * @param false $checkout
     * @return array
     * @throws Exception
     */
    private function getDeliveryData($checkout = false)
    {
        $address = new Address((int) $this->cart->id_address_delivery);

        $countryName = self::DEFAULT_COUNTRY;
        if (!empty($address->country)) {
            $countryName = $address->country;
        }

        $street = $address->address1;
        if(!empty($address->address2)) {
            $street.= ", " . $address->address2;
        }

        // Product count
        /*$productsCount = array_reduce($this->cart->getProducts(), function ($count, $product) {
            return $count + $product['quantity'];
        }, 0);*/

        // API call data
        $delivery =  array(
            "use_default_from_address" => "true",
            "type" => Configuration::get('CM_PACKAGING_TYPE'),
            "service_type" => Configuration::get('CM_DELIVERY_TYPE'),
            "cnt" => 1,//$productsCount, we have standard to 1
            "weight" => $this->cart->getTotalWeight(),
            "to_country" => $countryName,
            "to_county" => State::getNameById($address->id_state),
            "to_city" => $address->city,
            "to_address" => $street,
            "to_zipcode" => $address->postcode
        );

        if($checkout) {
            $customer = new Customer((int) $this->cart->id_customer);
            $name = $address->firstname ." " . $address->lastname;

            // API call data
            $delivery = array_merge($delivery, array(
                "to_name" => $name,
                "to_contact" => $name,
                "to_phone" => empty($address->phone)?$address->phone_mobile:$address->phone,
                "to_email" => $customer->email,
                "payer" => "client"
            ));

            // Add cash on delivery
            $cashOnDelivery = array("ps_cashondelivery", "cashondelivery");
            if (in_array($this->order->module, $cashOnDelivery)) {

                if (Configuration::get('CM_PAYMENT_AWB') == 1) {
                    $delivery["ramburs"] = number_format($this->order->total_products_wt, 2, '.', '');
                    $delivery["payer"] = "destinatar";
                }else{
                    $delivery["ramburs"] = number_format($this->order->total_paid_tax_incl, 2, '.', '');
                    $delivery["payer"] = "client";
                }

                $delivery["ramburs_type"] = Configuration::get('CM_COD_TYPE');
            }
        }else{
            $delivery["ramburs"] = number_format($this->cart->getOrderTotal(true, Cart::BOTH_WITHOUT_SHIPPING), 2, '.', '');
            $delivery["ramburs_type"] = Configuration::get('CM_COD_TYPE');
            $delivery["payer"] = "destinatar";
        }

        return $delivery;
    }

    /**
     * @return false|int|string|null
     * @throws Exception
     */
    private function getShippingCost()
    {
        // Check if total is upper minim free sum
        if (is_numeric(Configuration::get('CM_MINIMUM_AMOUNT')) && $this->cart->getOrderTotal(true, Cart::BOTH_WITHOUT_SHIPPING) > Configuration::get('CM_MINIMUM_AMOUNT')) {
            return 0;
        }

        // Return fix amount if is set
        if (is_numeric(Configuration::get('CM_FIX_AMOUNT')) && Configuration::get('CM_FIX_AMOUNT') > 0) {
            return Configuration::get('CM_FIX_AMOUNT');
        }

        // Call API
        $deliveryData = $this->getDeliveryData();
        $apiPrice = $this->apiClient->priceAwb($deliveryData);

        // Check if response is ok
        if (!isset($apiPrice->status) || $apiPrice->status != "done") {
            if (_PS_VERSION_ >= 1.6) {
                PrestaShopLogger::addLog($this->displayName  . ": [ShippingCost] " . $this->apiClient->getError(), 2, null, null, null, true);
            }

            return null;
        }

        // Return delivery price
        return $apiPrice->data->price;
    }

    /**
     * @param $cart
     * @param $shipping_cost
     * @return false|int|string|null
     * @throws Exception
     */
    public function getOrderShippingCost($cart, $shipping_cost)
    {
        $this->cart = $cart;
        return $this->getShippingCost();
    }

    /**
     * @param $cart
     * @return false|int|string|null
     * @throws Exception
     */
    public function getOrderShippingCostExternal($cart)
    {
        $this->cart = $cart;
        return $this->getShippingCost();
    }

    /**
     * @param $params
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public function hookActionOrderStatusUpdate($params)
    {
        // Allow only new order
        if(empty($params['newOrderStatus'])) {
            return;
		}

        // Get order
        $this->order = new Order((int) $params['id_order']);

        // Generate AWB
        foreach ($this->order->getShipping() as $shipping) {
            $carrier = new Carrier((int)$shipping['id_carrier']);
            if ($carrier->external_module_name != $this->name) {
                continue;
            }

            // Check if we have AWB
            if (!empty($shipping['tracking_number'])) {
                return;
            }

            // Set module id
            Configuration::updateValue('COURIERMANAGER_MODULE', (int) $shipping['id_carrier']);

            // Set cart
            $this->cart = $params['cart'];

            /** Call API */
            $deliveryData = $this->getDeliveryData(true);
            $apiResponse = $this->apiClient->createAwb($deliveryData);

            // Log error
            if (_PS_VERSION_ >= 1.6 && Configuration::get('DEBUG') == 1) {
                $debugLog = array(
                    'orderModule' => $this->order->module,
                    'deliveryData' => $deliveryData,
                    'apiResponse' => $apiResponse,
                );
                PrestaShopLogger::addLog($this->displayName  . ": [Debug][Order] " . base64_encode(json_encode($debugLog)), 1, null, null, null, true);
            }

            if (isset($apiResponse->status) && $apiResponse->status == "done" && isset($apiResponse->data->no)) {
                Db::getInstance()->execute('UPDATE '._DB_PREFIX_.'order_carrier set tracking_number = \''.$apiResponse->data->no.'\' where id_order = ' . $this->order->id);
                return;
            }

            // Log error
            if (_PS_VERSION_ >= 1.6) {
                PrestaShopLogger::addLog($this->displayName  . ": [Order] " . $this->apiClient->getError(), 2, null, null, null, true);
            }
        }
    }

    /**
     * @return false|string
     */
    public function getContent()
    {
        $output = null;

        if (Tools::isSubmit('submit'.$this->name)) {
            Configuration::updateValue('CM_API_URL', Tools::getValue('CM_API_URL'));
            Configuration::updateValue('CM_API_KEY', Tools::getValue('CM_API_KEY'));
            Configuration::updateValue('CM_DELIVERY_TYPE', Tools::getValue('CM_DELIVERY_TYPE'));
            Configuration::updateValue('CM_PACKAGING_TYPE', Tools::getValue('CM_PACKAGING_TYPE'));
            Configuration::updateValue('CM_PAYMENT_AWB', Tools::getValue('CM_PAYMENT_AWB'));
            Configuration::updateValue('CM_COD_TYPE', Tools::getValue('CM_COD_TYPE'));
            Configuration::updateValue('CM_GOODS_INSURED', Tools::getValue('CM_GOODS_INSURED'));
            Configuration::updateValue('CM_MINIMUM_AMOUNT', Tools::getValue('CM_MINIMUM_AMOUNT'));
            Configuration::updateValue('CM_FIX_AMOUNT', Tools::getValue('CM_FIX_AMOUNT'));
            Configuration::updateValue('DEBUG', Tools::getValue('DEBUG'));

            // Check API
            $apiClient = new CourierManagerApi(Configuration::get('CM_API_URL'), Configuration::get('CM_API_KEY'));
            $apiClient->getCityList();
            if($apiClient->getError()) {

                if (_PS_VERSION_ >= 1.6) {
                    PrestaShopLogger::addLog($this->displayName  . ": [Admin] " . $apiClient->getError(), 2, null, null, null, true);
                }

                Configuration::updateValue('CM_API_KEY', '');
                $output = $this->displayError($apiClient->getError());
            }
        }

        if (_PS_VERSION_ < 1.5) {
            return $this->displayOldForm();
        }

        return $output.$this->displayForm();
    }

    /**
     * @return false|string
     */
    private function displayOldForm()
    {
        global $smarty;

        $globalVar = array(
            'tab' => htmlentities(Tools::getValue('tab')),
            'configure' => htmlentities(Tools::getValue('configure')),
            'token' => htmlentities(Tools::getValue('token')),
            'tab_module' => htmlentities(Tools::getValue('tab_module')),
            'module_name' => htmlentities(Tools::getValue('module_name')));
        $smarty->assign('glob', $globalVar);

        $var = array(
            'CM_API_URL' => Tools::getValue('CM_API_URL', Configuration::get('CM_API_URL')),
            'CM_API_KEY' => Tools::getValue('CM_API_KEY', Configuration::get('CM_API_KEY')),
            'CM_DELIVERY_TYPE' => Tools::getValue('CM_DELIVERY_TYPE', Configuration::get('CM_DELIVERY_TYPE')),
            'CM_PACKAGING_TYPE' => Tools::getValue('CM_PACKAGING_TYPE', Configuration::get('CM_PACKAGING_TYPE')),
            'CM_PAYMENT_AWB' => Tools::getValue('CM_PAYMENT_AWB', Configuration::get('CM_PAYMENT_AWB')),
            'CM_COD_TYPE' => Tools::getValue('CM_COD_TYPE', Configuration::get('CM_COD_TYPE')),
            'CM_GOODS_INSURED' => Tools::getValue('CM_GOODS_INSURED', Configuration::get('CM_GOODS_INSURED')),
            'CM_MINIMUM_AMOUNT' => Tools::getValue('CM_MINIMUM_AMOUNT', Configuration::get('CM_MINIMUM_AMOUNT')),
            'CM_FIX_AMOUNT' => Tools::getValue('CM_FIX_AMOUNT', Configuration::get('CM_FIX_AMOUNT')),
            'DEBUG' => Tools::getValue('DEBUG', Configuration::get('DEBUG')),
        );
        $smarty->assign('varAccount', $var);
        return $this->display( __FILE__, 'tpl/oldForm.tpl' );
    }

    /**
     * @return string
     */
    public function displayForm()
    {
        // Get default language
        $defaultLang = (int)Configuration::get('PS_LANG_DEFAULT');

        // Init Fields form array
        $fieldsForm[0]['form'] = array(
            'legend' => array(
                'title' => $this->l('Settings'),
            ),
            'input' => array(
                array(
                    'type' => 'text',
                    'label' => $this->l('Api URL'),
                    'name' => 'CM_API_URL',
                    'size' => 20,
                    'required' => true
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Api Key'),
                    'name' => 'CM_API_KEY',
                    'size' => 20,
                    'required' => true
                )
            ),
            'submit' =>  array(
                'title' => $this->l('Save'),
                'class' => 'btn btn-default pull-right'
            )
        );

        if (Configuration::get('CM_API_KEY')) {
            // Get services
            $apiClient = new CourierManagerApi(Configuration::get('CM_API_URL'), Configuration::get('CM_API_KEY'));
            $services = array();
            foreach($apiClient->getServicesList() as $service) {
                $services[] = array(
                    'id_option' => $service->value,
                    'name' => $service->name,
                );
            }

            $fieldsForm[0]['form']['input'] = array_merge($fieldsForm[0]['form']['input'], array(
                array(
                    'type' => 'select',
                    'label' => $this->l('Delivery type code'),
                    'name' => 'CM_DELIVERY_TYPE',
                    'options' => array(
                        'query' => $services,
                        'id' => 'id_option',
                        'name' => 'name'
                    ),
                    'required' => true
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('Packaging type'),
                    'name' => 'CM_PACKAGING_TYPE',
                    'options' => array(
                        'query' => array(
                            array(
                                'id_option' => 'plic',
                                'name' => 'Envelope'
                            ),
                            array(
                                'id_option' => 'colet',
                                'name' => 'Package'
                            ),
                        ),
                        'id' => 'id_option',
                        'name' => 'name'
                    ),
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('Payment AWB at destination'),
                    'name' => 'CM_PAYMENT_AWB',
                    'options' => array(
                        'query' => array(
                            array(
                                'id_option' => 1,
                                'name' => 'YES'
                            ),
                            array(
                                'id_option' => 0,
                                'name' => 'NO'
                            ),
                        ),
                        'id' => 'id_option',
                        'name' => 'name'
                    ),
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('COD type'),
                    'name' => 'CM_COD_TYPE',
                    'options' => array(
                        'query' => array(
                            array(
                                'id_option' => 'cont',
                                'name' => 'Bank account'
                            ),
                            array(
                                'id_option' => 'cash',
                                'name' => 'Cash'
                            ),
                        ),
                        'id' => 'id_option',
                        'name' => 'name'
                    ),
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('Goods Insured'),
                    'name' => 'CM_GOODS_INSURED',
                    'options' => array(
                        'query' => array(
                            array(
                                'id_option' => 1,
                                'name' => 'YES'
                            ),
                            array(
                                'id_option' => 0,
                                'name' => 'NO'
                            ),
                        ),
                        'id' => 'id_option',
                        'name' => 'name'
                    ),
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Minimum amount of free shipping'),
                    'name' => 'CM_MINIMUM_AMOUNT',
                    'size' => 20,
                ),
                array(
                    'type' => 'text',
                    'label' => $this->l('Fix amount of shipping'),
                    'name' => 'CM_FIX_AMOUNT',
                    'size' => 20,
                ),
                array(
                    'type' => 'select',
                    'label' => $this->l('Debug'),
                    'name' => 'DEBUG',
                    'options' => array(
                        'query' => array(
                            array(
                                'id' => 1,
                                'name' => 'YES'
                            ),
                            array(
                                'id' => 0,
                                'name' => 'NO'
                            ),
                        ),
                        'id' => 'id',
                        'name' => 'name'
                    ),
                ),
            ));
        }

        $helper = new HelperForm();

        // Module, token and currentIndex
        $helper->module = $this;
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;

        // Language
        $helper->default_form_language = $defaultLang;
        $helper->allow_employee_form_lang = $defaultLang;

        // Title and toolbar
        $helper->title = $this->displayName;
        $helper->show_toolbar = true;        // false -> remove toolbar
        $helper->toolbar_scroll = true;      // yes - > Toolbar is always visible on the top of the screen.
        $helper->submit_action = 'submit'.$this->name;
        $helper->toolbar_btn = array(
            'save' => array(
                'desc' => $this->l('Save'),
                'href' => AdminController::$currentIndex.'&configure='.$this->name.'&save'.$this->name.
                    '&token='.Tools::getAdminTokenLite('AdminModules'),
            ),
            'back' => array(
                'href' => AdminController::$currentIndex.'&token='.Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('Back to list')
            )
        );

        // Load current value
        $helper->fields_value['CM_API_URL'] = Configuration::get('CM_API_URL');
        $helper->fields_value['CM_API_KEY'] = Configuration::get('CM_API_KEY');
        $helper->fields_value['CM_DELIVERY_TYPE'] = Configuration::get('CM_DELIVERY_TYPE');
        $helper->fields_value['CM_PACKAGING_TYPE'] = Configuration::get('CM_PACKAGING_TYPE');
        $helper->fields_value['CM_PAYMENT_AWB'] = Configuration::get('CM_PAYMENT_AWB');
        $helper->fields_value['CM_COD_TYPE'] = Configuration::get('CM_COD_TYPE');
        $helper->fields_value['CM_GOODS_INSURED'] = Configuration::get('CM_GOODS_INSURED');
        $helper->fields_value['CM_MINIMUM_AMOUNT'] = Configuration::get('CM_MINIMUM_AMOUNT');
        $helper->fields_value['CM_FIX_AMOUNT'] = Configuration::get('CM_FIX_AMOUNT');
        $helper->fields_value['DEBUG'] = Configuration::get('DEBUG');

        return $helper->generateForm($fieldsForm);
    }

    /**
     * @param $config
     * @return false|int
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public function installExternalCarrier($config)
    {
        $carrier = new Carrier();
        $carrier->name = $config['name'];
        $carrier->id_tax_rules_group = $config['id_tax_rules_group'];
        $carrier->id_zone = $config['id_zone'];
        $carrier->active = $config['active'];
        $carrier->deleted = $config['deleted'];
        $carrier->delay = $config['delay'];
        $carrier->shipping_handling = $config['shipping_handling'];
        $carrier->range_behavior = $config['range_behavior'];
        $carrier->is_module = $config['is_module'];
        $carrier->shipping_external = $config['shipping_external'];
        $carrier->external_module_name = $config['external_module_name'];
        $carrier->need_range = $config['need_range'];
        $languages = Language::getLanguages(true);
        foreach ($languages as $language) {
            if ($language['iso_code'] == 'fr')
                $carrier->delay[(int) $language['id_lang']] = $config['delay'][$language['iso_code']];
            if ($language['iso_code'] == 'en')
                $carrier->delay[(int) $language['id_lang']] = $config['delay'][$language['iso_code']];
            if ($language['iso_code'] == Language::getIsoById(Configuration::get('PS_LANG_DEFAULT')))
                $carrier->delay[(int) $language['id_lang']] = $config['delay'][$language['iso_code']];
        }
        if ($carrier->add()) {
            $groups = Group::getGroups(true);
            foreach ($groups as $group) {
                Db::getInstance()->execute('INSERT INTO ' . _DB_PREFIX_ . 'carrier_group VALUE (\'' . (int)($carrier->id) . '\',\'' . (int)($group['id_group']) . '\')');
            }
            $rangePrice = new RangePrice();
            $rangePrice->id_carrier = $carrier->id;
            $rangePrice->delimiter1 = '0';
            $rangePrice->delimiter2 = '10000';
            $rangePrice->add();
            $rangeWeight = new RangeWeight();
            $rangeWeight->id_carrier = $carrier->id;
            $rangeWeight->delimiter1 = '0';
            $rangeWeight->delimiter2 = '10000';
            $rangeWeight->add();
            $zones = Zone::getZones(true);
            foreach ($zones as $zone) {
                Db::getInstance()->execute('INSERT INTO '._DB_PREFIX_.'carrier_zone  (id_carrier, id_zone) VALUE (\''.(int)($carrier->id).'\',\''.(int)($zone['id_zone']).'\')');
                Db::getInstance()->execute('INSERT INTO '._DB_PREFIX_.'delivery (id_carrier, id_range_price, id_range_weight, id_zone, price) VALUE (\''.(int)($carrier->id).'\',\''.(int)($rangePrice->id).'\',NULL,\''.(int)($zone['id_zone']).'\',\'0\')');
                Db::getInstance()->execute('INSERT INTO '._DB_PREFIX_.'delivery (id_carrier, id_range_price, id_range_weight, id_zone, price) VALUE (\''.(int)($carrier->id).'\',NULL,\''.(int)($rangeWeight->id).'\',\''.(int)($zone['id_zone']).'\',\'0\')');
            }
            // Copy Logo
            if (!copy(dirname(__FILE__) . '/logo.png', _PS_SHIP_IMG_DIR_ . '/' . (int) $carrier->id . '.jpg'))
                return false;
            // Return ID Carrier
            return (int) ($carrier->id);
        }
        return false;
    }
}
