<?php
/*
Plugin Name: Courier Manager
Plugin URI: https://app.couriermanager.eu/
Description: Courier Manager delivery module
Version: 1.3
Author: Alex Stoica
Author URI: https://devgeeks.ro
*/

global $jal_db_version;

use Automattic\WooCommerce\Admin\Overrides\Order;

$couriermanager_version = '1.3';

require_once (__DIR__ . '/functions/Helpers.php');
require_once (__DIR__ . '/functions/Install.php');

/**
 * Check if WooCommerce is active
 */

if (isWoocommerceActive()) {
    require_once (__dir__ . "/services/CourierManagerApi.php");

    /**
     * Shipping method init
     */
    function couriermanager_shipping_method_init() {
        if ( ! class_exists( 'WC_CourierManager_Shipping_Method' ) ) {
            require_once (__dir__ . "/hooks/ShippingMethod.php");
        }
    }
    add_action( 'woocommerce_shipping_init', 'couriermanager_shipping_method_init' );

    /**
     * Add shipping method to cart
     * @param $methods
     * @return mixed
     */
    function add_couriermanager_shipping_method( $methods ) {
        $methods['couriermanager_shipping_method'] = 'WC_CourierManager_Shipping_Method';
        return $methods;
    }
    add_filter( 'woocommerce_shipping_methods', 'add_couriermanager_shipping_method' );


    /**
     * @param $order_id
     */
    function couriermanager_finish_order($order_id) {
        require_once (__dir__ . "/hooks/CreateAwb.php");
        new CreateAwb(new WC_Order($order_id));
    }
    add_filter( 'woocommerce_checkout_order_processed', 'couriermanager_finish_order',  1, 1 );

    function couriermanager_refresh_checkout()
    {
        $packages = WC()->cart->get_shipping_packages();
        foreach ($packages as $package_key => $package ) {
            WC()->session->set( 'shipping_for_package_' . $package_key, false ); // Or true
        }
    }
    add_action('woocommerce_checkout_update_order_review', 'couriermanager_refresh_checkout');

    function couriermanager_refresh_checkout_script() {
        if ( is_checkout() && ! is_wc_endpoint_url() ) {
            require_once (__DIR__ . "/views/checkout-refresh.php");
        }
    }
    add_action('wp_footer', 'couriermanager_refresh_checkout_script');

	/**
	 * @param Order $order
	 */
	function couriermanager_manual_order_delivery($order) {
		global $wpdb;

		if ($_GET['action'] !== 'edit' || $order->get_meta('couriermanager_awb')) {
			return;
		}

		$shippingMethods = WC_Shipping::instance()->get_shipping_methods();
		if (!isset($shippingMethods['couriermanager_shipping_method'])) {
			return;
		}
		$shoppingMethod = $shippingMethods['couriermanager_shipping_method'];

		$query = "SELECT z.zone_name, m.*
					FROM {$wpdb->prefix}woocommerce_shipping_zone_methods m
					LEFT JOIN {$wpdb->prefix}woocommerce_shipping_zones z on z.zone_id = m.zone_id
					WHERE method_id = 'couriermanager_shipping_method' AND is_enabled = 1";
		$zoneShippingMethods = $wpdb->get_results($query);

		if (empty($zoneShippingMethods)) {
			return;
		}

		$cmShippingMethods = array();
		foreach($zoneShippingMethods as $method) {
			$shoppingMethod->instance_id = $method->instance_id;
			$cmShippingMethods[] = array(
				'instance_id' => $method->instance_id,
				'zone_name' => $method->zone_name === null ? "Global":$method->zone_name,
				'title' => $shoppingMethod->get_instance_option('title')
			);
		}

		require_once (__DIR__ . "/views/order.php");
	}
	add_action( 'woocommerce_admin_order_data_after_shipping_address', 'couriermanager_manual_order_delivery');


	/**
	 * @param $order_id
	 */
	function couriermanager_manual_order_delivery_save( $order_id ) {
		require_once (__DIR__ . "/hooks/CreateAwb.php");

		$order = wc_get_order( $order_id );
		if (!isset($_POST['couriermanager-awb-method']) || $order->get_meta('couriermanager_awb')) {
			return;
		}

		$shippingMethods = WC_Shipping::instance()->get_shipping_methods();
		if (!isset($shippingMethods['couriermanager_shipping_method'])) {
			return;
		}

		$shoppingMethod = $shippingMethods['couriermanager_shipping_method'];
		$shoppingMethod->instance_id = $_POST['couriermanager-awb-method'];

		new CreateAwb($order, $shoppingMethod, CreateAwb::CONTEXT_ORDER);
	}
	add_action( 'woocommerce_update_order', 'couriermanager_manual_order_delivery_save', 10, 1 );
}