<?php

class WC_CourierManager_Shipping_Method extends WC_Shipping_Method
{
	const SHIPPING_METHOD_ID = "couriermanager_shipping_method";
    const DEFAULT_COUNTRY = "RO";
    const DEFAULT_COUNTY = "B";

    /** @var CourierManagerApi */
    protected $apiClient;

    /**
     * Constructor for your shipping class
     *
     * @access public
     * @param int $instance_id
     */
    public function __construct($instance_id = 0)
    {
        $this->id                 = self::SHIPPING_METHOD_ID; // Id for your shipping method. Should be uunique.
        $this->instance_id        = absint( $instance_id );
        $this->method_title       = __( 'Courier Manager' );  // Title shown in admin
        $this->method_description = __( 'Courier software that just works!' ); // Description shown in admin
        $this->title              = "Courier Manager"; // This can be added as an setting but for this example its forced.

        $this->supports = array(
            'shipping-zones',
            'instance-settings',
            'instance-settings-modal',
        );

        $this->init();
    }

    /**
     * Init your settings
     *
     * @access public
     * @return void
     */
    public function init()
    {
        $this->init_settings();
        $this->init_instance_form_fields();

        $this->title  = $this->get_option( 'title' );

	    add_action( 'woocommerce_update_options_shipping_' . $this->id, array( $this, 'process_admin_options' ) );
    }

    private function init_instance_form_fields()
    {
        $this->instance_form_fields+= array(
        	'title' => array(
		        'title' => __('Title *'),
		        'type' => 'text',
		        'default' => __('Courier Manager'),
		        'required' => true,
	        ),
            'api_label' => array(
                'title' => __('Api: '),
                'type' => 'title',
            ),

            'api_url' => array(
                'title' => __('Api URL *'),
                'type' => 'text',
                'default' => __('https://app.couriermanager.eu/cscourier'),
                'required' => true,
            ),

            'api_key' => array(
                'title' => __('Api Key *'),
                'type' => 'text',
                'default' => '',
                'required' => true,

            )
        );

        if ($this->get_option('api_url') && $this->get_option('api_key')) {
            // Check API
            $deliveryTypeCodes = array();
            $deliveryServices = $this->getClient()->getServicesList();
            if(!$this->getClient()->getError()) {
                foreach ($deliveryServices as $service) {
                    $deliveryTypeCodes[$service->value] = $service->name;
                }
            }

            $this->instance_form_fields += array(
                'delivery_label' => array(
                    'title' => __('Delivery: '),
                    'type' => 'title',
                ),

                'shipping_vat' => array(
                    'title' => __('Shipping VAT *'),
                    'type' => 'number',
                    'default' => '19',

                ),

                'delivery_type_code' => array(
                    'title' => __('Delivery type code *'),
                    'type' => 'select',
                    'label' => '',
                    'css' => 'width:350px;',
                    'options' => $deliveryTypeCodes,
                    'required' => true,
                ),

                'packaging_type' => array(
                    'title' => __('Packaging type'),
                    'type' => 'select',
                    'label' => '',
                    'default' => 'colet',
                    'css' => 'width:350px;',
                    'options' => array(
                        'plic' => __('Envelope'),
                        'colet' => __('Package')
                    ),

                ),

                'payment_awb' => array(
                    'title' => __('Payment AWB at destination'),
                    'type' => 'select',
                    'label' => '',
                    'default' => '0',
                    'css' => 'width:350px;',
                    'options' => array(
                        '0' => __('No'),
                        '1' => __('Yes')
                    ),
                ),

                'cod_type' => array(
                    'title' => __('COD type'),
                    'type' => 'select',
                    'label' => '',
                    'default' => 'cont',
                    'css' => 'width:350px;',
                    'options' => array(
                        'cont' => __('Bank account'),
                        'cash' => __('Cash')
                    ),
                ),

                'goods_insured' => array(
                    'title' => __('Goods Insured'),
                    'type' => 'select',
                    'label' => '',
                    'default' => '0',
                    'css' => 'width:350px;',
                    'options' => array(
                        '0' => __('No'),
                        '1' => __('Yes')
                    ),
                ),

	            'auto_awb' => array(
		            'title' => __('Automatic processing AWB'),
		            'type' => 'checkbox',
		            'label' => __('Yes'),
		            'default' => 'yes',
		            'description' => "If you disable this option you will need to send AWB manually (you will find a button in each order)",
	            ),

                'delivery_price_label' => array(
                    'title' => __('Amount rules: '),
                    'type' => 'title',
                ),

                'minim_amount' => array(
                    'title' => __('Minimum amount of free shipping'),
                    'type' => 'price'
                ),

                'fix_amount' => array(
                    'title' => __('Fix amount of shipping'),
                    'type' => 'price'
                )
            );
        }
    }

    public function process_admin_options()
    {
	    $post_data = $this->get_post_data();

	    // Check API
	    $apiClient = new CourierManagerApi(
		    $post_data['woocommerce_couriermanager_shipping_method_api_url'],
		    $post_data['woocommerce_couriermanager_shipping_method_api_key']
	    );
	    $apiClient->getCityList();
	    if($apiClient->getError()) {
		    $this->add_error($apiClient->getError());
		    return false;
	    }

	    parent::process_admin_options();
    }

    /**
     * @param array $package
     * @return array
     */
    private function getDeliveryData($package = array())
    {
        $countryStates = WC()->countries->get_shipping_country_states();
        $countries = WC()->countries->get_shipping_countries();
        $countryId = $package['destination']['country'];
        $stateId = $package['destination']['state'];

        $countryName = self::DEFAULT_COUNTRY;
        if (!empty($countries[$countryId])) {
            $countryName = $countries[$countryId];
        }

        $regionName = self::DEFAULT_COUNTY;
        if (!empty($countryStates[$countryId][$stateId])) {
            $regionName = $countryStates[$countryId][$stateId];
        }

        // Address
        $address = $package['destination']['address'];
        if (!empty($package['destination']['address_1'])) {
            $address.= ", " . $package['destination']['address_1'];
        }

        if (!empty($package['destination']['address_2'])) {
            $address.= ", " . $package['destination']['address_2'];
        }

        $cartPrice = wc_prices_include_tax() ? WC()->cart->get_cart_contents_total() + WC()->cart->get_cart_contents_tax() : WC()->cart->get_cart_contents_total();

        $weight = WC()->cart->get_cart_contents_weight();
        switch (get_option('woocommerce_weight_unit')) {
            case "g":
                $weight = ceil($weight / 100);
                break;
            case "lbs":
                $weight = ceil($weight * 0.45359);
                break;
        }

        // API call data
        $apiData = array(
            "use_default_from_address" => "true",
            "type" => $this->get_option('packaging_type'),
            "service_type" => $this->get_option('delivery_type_code'),
            "cnt" => 1,//WC()->cart->get_cart_contents_count(),
            "weight" => $weight,
            "to_country" => $countryName,
            "to_county" => $regionName,
            "to_city" => $package['destination']['city'],
            "to_address" => $address,
            "to_zipcode" => $package['destination']['postcode'],
        );

        if (isset($_POST['payment_method']) && $_POST['payment_method'] === "cod") {
            $apiData['ramburs'] = $cartPrice;
            $apiData['ramburs_type'] = $this->get_option('cod_type');
        }

        return $apiData;
    }

    /**
     * @param array $package
     * @return int|mixed|null
     */
    private function getShippingCost($package = array())
    {
        // Check if total is upper minim free sum
        if ($this->get_option('minim_amount') && WC()->cart->get_subtotal() >= $this->get_option('minim_amount')) {
            return 0;
        }

        // Return fix amount if is set
        if ($this->get_option('fix_amount') && $this->get_option('fix_amount') > 0) {
            return $this->get_option('fix_amount');
        }

        // Call API
        $deliveryData = $this->getDeliveryData($package);
        $apiPrice = $this->getClient()->priceAwb($deliveryData);

        // Check if response is ok
        if (!isset($apiPrice->status) || $apiPrice->status != "done") {
            return null;
        }

        $taxes = $apiPrice->data->price * (  $this->get_option('shipping_vat') / 100 );

        // Return delivery price
        return $apiPrice->data->price + $taxes;
    }

	/**
	 * @return CourierManagerApi
	 */
    private function getClient()
    {
    	if (!is_null($this->apiClient)){
    		return $this->apiClient;
	    }

	    if (!$this->get_option('api_url') || !$this->get_option('api_key')) {
		    return new CourierManagerApi(null, null);
	    }

	    $this->apiClient = new CourierManagerApi($this->get_option('api_url'), $this->get_option('api_key'));
	    return $this->apiClient;
    }

    /**
     * calculate_shipping function.
     *
     * @access public
     * @param mixed $package
     * @return void
     */
    public function calculate_shipping( $package = array() )
    {
    	$shippingCost = $this->getShippingCost($package);
    	if (is_null($shippingCost)) {
    		return;
	    }

        $rate = array(
            'id' => $this->get_rate_id('couriermanager'),
            'label' => $this->get_option('title'),
            'cost' => $shippingCost,
            'calc_tax' => 'per_order',
        );

        // Register the rate
        $this->add_rate( $rate );
    }
}
