<?php

require_once DIR_SYSTEM . 'library/CourierManagerApi.php';

class ControllerExtensionShippingCouriermanager extends Controller
{
    const KG_CLASS_ID = 1;

    private $error = array();

    public function install()
    {
        $this->load->model('extension/shipping/couriermanager');
        $this->model_extension_shipping_couriermanager->install();

        $this->load->model('setting/setting');
        $this->model_setting_setting->editSetting($this->getPrefix() . "couriermanager",
            array(
                $this->getPrefix() . "couriermanager_title" => "Courier Manager",
                $this->getPrefix() . "couriermanager_apiurl" => "https://app.couriermanager.eu/cscourier"
            )
        );
    }

    public function uninstall()
    {
        $this->load->model('extension/shipping/couriermanager');
        $this->model_extension_shipping_couriermanager->uninstall();
    }


    public function index() {
        $this->load->language('extension/shipping/couriermanager');
        $this->document->setTitle($this->language->get('heading_title'));
        $this->load->model('setting/setting');

        if (($this->request->server['REQUEST_METHOD'] == 'POST') && $this->validate()) {
            $this->model_setting_setting->editSetting($this->getPrefix() . "couriermanager", $this->request->post);
            $this->session->data['success'] = "The data has been saved.";
            $this->response->redirect($this->url->link('extension/shipping/couriermanager', $this->addToken(), true));
        }

        $data = array();
        $data['breadcrumbs'] = array();
        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('text_home'),
            'href' => $this->url->link('common/dashboard', $this->addToken(), true)
        );

        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('text_extension'),
            'href' => $this->url->link($this->getRouteExtension(), $this->addToken(array('type' => 'shipping')), true)
        );

        $data['breadcrumbs'][] = array(
            'text' => $this->language->get('heading_title'),
            'href' => $this->url->link('extension/shipping/couriermanager', $this->addToken(), true)
        );

        $data['action'] = $this->url->link('extension/shipping/couriermanager', $this->addToken(), true);
        $data['cancel'] = $this->url->link($this->getRouteExtension(), $this->addToken(array('type' => 'shipping')), true);

        $data['heading_title'] = $this->language->get('heading_title');

        // Form vars
        $forms = array(
            "title", "apiurl", "apikey", "shipping_vat", "delivery_type",
            "packing_type", "payment_awb", "cod_type", "goods_insured",
            "minimum_amount", "fix_amount", "sort_order", "status"
        );
        foreach ($forms as $form) {
            $data[$this->getPrefix() . "couriermanager_" . $form] = $this->getConfig($form);
        }

        // Send post vars to view
        if (!empty($this->request->post)) {
            foreach ($this->request->post as $key => $value) {
                $data[$key] = $value;
            }
        }

        if ($this->getConfig('apikey')) {
            $data["services"] = $this->client()->getServicesList();
        }

        $data['error'] = $this->error;
        $data['header'] = $this->load->controller('common/header');
        $data['column_left'] = $this->load->controller('common/column_left');
        $data['footer'] = $this->load->controller('common/footer');

        $this->response->setOutput($this->load->view('extension/shipping/couriermanager', $data));
    }

    public function validate()
    {
        if (!$this->user->hasPermission('modify', 'extension/shipping/couriermanager')) {
            $this->error['warning'] = $this->language->get('error_permission');
        }

        if (!$this->getPost('title')) {
            $this->error['title'] = "Please fill title";
        }

        if (!$this->getPost('apiurl')) {
            $this->error['api_url'] = "Please fill the API URL";
        }

        if (!$this->getPost('apikey')) {
            $this->error['api_key'] = "Please fill the API KEY";
        }

        if($this->getPost('apiurl') && $this->getPost('apikey')){
            $apiClient = $this->client($this->getPost('apiurl'), $this->getPost('apikey'));
            $apiClient->getCityList();
            if($apiClient->getError()) {
                $this->setPost("apikey", null);
                $this->error['warning'] = $apiClient->getError();
            } else if (!$this->getConfig('apikey')) {
                return !$this->error;
            }
        }

        if (!$this->getConfig('apiurl') || !$this->getConfig('apiurl')) {
            return !$this->error;
        }

        if (!$this->getPost('delivery_type')) {
            $this->error['delivery_type'] = "Please fill the Delivery type";
        }

        if ($this->getPost('shipping_vat') == "" || !is_numeric($this->getPost('shipping_vat')) ||
            $this->getPost('shipping_vat') < 0) {
            $this->error['shipping_vat'] = "Please fill the Shipping VAT";
        }

        return !$this->error;
    }

    public function info($orderInfo)
    {
        if (!$orderInfo) {
            return null;
        }
        $this->load->model('extension/shipping/couriermanager');

        $data = array(
            'awbName' => "{$this->getConfig('title')} AWB",
            'generateAwbLink' => $this->url->link('extension/shipping/couriermanager/addAwb', $this->addToken(array('order_id' => $orderInfo['order_id'])), true)
        );

        $awb = $this->model_extension_shipping_couriermanager->getAwbByOrderId($orderInfo['order_id']);
        if ($awb) {
            $data['awb'] = $awb['awb'];
        }

        return $data;
    }

    public function addAwb()
    {
        $this->load->model('sale/order');

        if (!isset($this->request->get['order_id']) ||
            !($orderInfo = $this->model_sale_order->getOrder($this->request->get['order_id']))) {
            return new Action('error/not_found');
        }

        $this->load->model('extension/shipping/couriermanager');
        $awb = $this->model_extension_shipping_couriermanager->getAwbByOrderId($orderInfo['order_id']);
        if ($awb) {
            $this->response->redirect($this->url->link('sale/order/info', $this->addToken(array('order_id' => $orderInfo['order_id'])), true));
        }

        $address = $orderInfo["shipping_address_1"];
        if(!empty($orderInfo["shipping_address_2"])) {
            $address.= ", " . $orderInfo["shipping_address_2"];
        }

        $name = $orderInfo["shipping_firstname"] . " " . $orderInfo["shipping_lastname"];

        // API call data
        $deliveryData =  array(
            "use_default_from_address" => "true",
            "type" => $this->getConfig('delivery_type'),
            "service_type" => $this->getConfig('packing_type'),
            "cnt" => 1,
            "weight" => $this->calculateWeight($orderInfo['order_id']),
            "to_country" => $orderInfo["shipping_country"],
            "to_county" => $orderInfo["shipping_zone"],
            "to_city" => $orderInfo["shipping_city"],
            "to_address" => $address,
            "to_zipcode" => $orderInfo["shipping_postcode"],
            "to_name" => $name,
            "to_contact" => $name,
            "to_phone" => $orderInfo["telephone"],
            "to_email" => $orderInfo["email"],
        );

        // Add cash on delivery
        if ($orderInfo["payment_code"] == "cod") {
            $deliveryData["ramburs"] = round($orderInfo['total'], 2);
            $deliveryData["ramburs_type"] = $this->getConfig('cod_type');
        }

        $apiResponse = $this->client()->createAwb($deliveryData);
        if (isset($apiResponse->status) && $apiResponse->status == "done" && isset($apiResponse->data->no)) {
            $taxes = $apiResponse->data->price * (  $this->getConfig('shipping_vat') / 100 );
            $this->model_extension_shipping_couriermanager->saveAwb([
                'order_id' => $orderInfo['order_id'],
                'awb' => $apiResponse->data->no,
                'price' => $apiResponse->data->price * $taxes
            ]);

            $this->response->redirect($this->url->link('sale/order/info', $this->addToken(array('order_id' => $orderInfo['order_id'])), true));
        }

        print_r($this->client()->getError());
    }

    private function calculateWeight($orderId)
    {
        $this->load->model('sale/order');
        $this->load->model('catalog/product');

        $weight = 0;
        $orderItems = $this->model_sale_order->getOrderProducts($orderId);
        foreach ($orderItems as $orderItem) {
            $product = $this->model_catalog_product->getProduct($orderItem['product_id']);
            $weight += ceil($this->weight->convert($product['weight'] * $orderItem['quantity'], $product['weight_class_id'], self::KG_CLASS_ID));
        }

        return $weight;
    }

    private function getPrefix()
    {
        if (strpos(VERSION, '2') !== false) {
            return '';
        }

        return 'shipping_';
    }

    private function getRouteExtension()
    {
        if (strpos(VERSION, '2') !== false) {
            return 'extension/extension';
        }

        return 'marketplace/extension';
    }

    /**
     * @param array $parts
     *
     * @return array
     */
    private function addToken(array $parts = array())
    {
        if (isset($this->session->data['token'])) {
            return array_merge($parts, array('token' => $this->session->data['token']));
        }

        if (isset($this->session->data['user_token'])) {
            return array_merge($parts, array('user_token' => $this->session->data['user_token']));
        }

        return $parts;
    }

    private function getPost($key)
    {
        if (!isset($this->request->post[$this->getPrefix() . "couriermanager_". $key])) {
            return null;
        }

        return $this->request->post[$this->getPrefix() . "couriermanager_". $key];
    }

    private function setPost($key, $value)
    {
        return $this->request->post[$this->getPrefix() . "couriermanager_". $key] = $value;
    }

    /**
     * @param string $key
     *
     * @return mixed
     */
    private function getConfig($key)
    {
        return $this->config->get($this->getPrefix() . "couriermanager_". $key);
    }


    /**
     * @param null $apiURL
     * @param null $apiKEY
     * @return CourierManagerApi
     */
    private function client($apiURL = null, $apiKEY = null)
    {
        if (!$apiURL && !$apiKEY) {
            $apiURL = $this->getConfig('apiurl');
            $apiKEY = $this->getConfig('apikey');
        }

        return new CourierManagerApi(
            $apiURL,
            $apiKEY
        );
    }
}
