<?php

require_once DIR_SYSTEM . 'library/CourierManagerApi.php';

class ModelExtensionShippingCouriermanager extends Model
{

    /** @var array */
    private $address;

    /**
     * @return array
     */
    private function getDeliveryData()
    {
        $address = $this->address["address_1"];
        if(!empty($address->address2)) {
            $address.= ", " . $this->address["address_2"];
        }

        // API call data
        $delivery =  array(
            "use_default_from_address" => "true",
            "type" => $this->getConfig('delivery_type'),
            "service_type" => $this->getConfig('packing_type'),
            "cnt" => 1,//$this->cart->countProducts(),
            "weight" => round($this->cart->getWeight()),
            "to_country" => $this->address["country"],
            "to_county" => $this->address["zone"],
            "to_city" => $this->address["city"],
            "to_address" => $address,
            "to_zipcode" => $this->address["postcode"],
            "ramburs" => $this->cart->getTotal(),
            "ramburs_type" => $this->getConfig('cod_type')
        );

        return $delivery;
    }

    /**
     * @return int|null
     */
    private function getShippingCost()
    {
        if (is_numeric($this->getConfig('minimum_amount')) &&
            $this->cart->getSubTotal() > $this->getConfig('minimum_amount')) {
            return 0;
        }

        // Return fix amount if is set
        if (is_numeric($this->getConfig('fix_amount')) &&
            $this->getConfig('fix_amount') > 0) {
            return $this->getConfig('fix_amount');
        }

        // Call API
        $deliveryData = $this->getDeliveryData();
        $apiPrice = $this->client()->priceAwb($deliveryData);

        // Check if response is ok
        if (!isset($apiPrice->status) || $apiPrice->status != "done") {
            return null;
        }

        $taxes = $apiPrice->data->price * (  $this->getConfig('shipping_vat') / 100 );

        // Return delivery price
        return $apiPrice->data->price + $taxes;
    }

    /**
     * @param array $address
     * @return array
     */
    function getQuote($address) {
        $this->load->language('extension/shipping/couriermanager');
        $this->address = $address;

        $quoteData = array( 'couriermanager' => array(
            'code'         => 'couriermanager.couriermanager',
            'title'        => $this->getConfig('title'),
            'cost'         => $this->getShippingCost(),
            'tax_class_id' => 0,
            'text'         => $this->currency->format($this->getShippingCost(), $this->session->data['currency'])
        ));

        return array(
            'code'       => 'couriermanager',
            'title'      => $this->getConfig('title'),
            'quote'      => $quoteData,
            'sort_order' => $this->getConfig('sort_order'),
            'error'      => false
        );
    }

    /**
     * @return string
     */
    private function getPrefix()
    {
        if (strpos(VERSION, '2') !== false) {
            return '';
        }

        return 'shipping_';
    }

    /**
     * @param string $key
     *
     * @return mixed
     */
    private function getConfig($key)
    {
        return $this->config->get($this->getPrefix() . "couriermanager_". $key);
    }


    /**
     * @return CourierManagerApi
     */
    private function client()
    {
        $apiURL = $this->getConfig('apiurl');
        $apiKEY = $this->getConfig('apikey');

        return new CourierManagerApi(
            $apiURL,
            $apiKEY
        );
    }
}
